@@ -1,4 +1,4 @@
-# type: ignore
+from __future__ import annotations
 
 from pygments.lexer import inherit
 from pygments.lexers.sql import MySqlLexer
