@@ -1,3 +1,5 @@
+# type: ignore
+
 from collections import defaultdict, namedtuple
 import logging
 import os
@@ -46,10 +48,11 @@ from mycli.key_bindings import mycli_bindings
 from mycli.lexer import MyCliLexer
 from mycli.packages import special
 from mycli.packages.filepaths import dir_path_exists, guess_socket_location
+from mycli.packages.hybrid_redirection import get_redirect_components, is_redirect_command
 from mycli.packages.parseutils import is_destructive, is_dropping_database
 from mycli.packages.prompt_utils import confirm, confirm_destructive_query
 from mycli.packages.special.favoritequeries import FavoriteQueries
-from mycli.packages.special.main import NO_QUERY
+from mycli.packages.special.main import ArgType
 from mycli.packages.tabular_output import sql_format
 from mycli.packages.toolkit.history import FileHistoryWithTimestamp
 from mycli.sqlcompleter import SQLCompleter
@@ -72,7 +75,7 @@ class PasswordFileError(Exception):
     """Base exception for errors related to reading password files."""
 
 
-class MyCli(object):
+class MyCli:
     default_prompt = "\\t \\u@\\h:\\d> "
     default_prompt_splitln = "\\u@\\h\\n(\\t):\\d>"
     max_len_prompt = 45
@@ -195,24 +198,24 @@ class MyCli(object):
         self.prompt_app = None
 
     def register_special_commands(self):
-        special.register_special_command(self.change_db, "use", "\\u", "Change to a new database.", aliases=("\\u",))
+        special.register_special_command(self.change_db, "use", "\\u", "Change to a new database.", aliases=["\\u"])
         special.register_special_command(
             self.change_db,
             "connect",
             "\\r",
             "Reconnect to the database. Optional database argument.",
-            aliases=("\\r",),
+            aliases=["\\r"],
             case_sensitive=True,
         )
         special.register_special_command(
-            self.refresh_completions, "rehash", "\\#", "Refresh auto-completions.", arg_type=NO_QUERY, aliases=("\\#",)
+            self.refresh_completions, "rehash", "\\#", "Refresh auto-completions.", arg_type=ArgType.NO_QUERY, aliases=["\\#"]
         )
         special.register_special_command(
             self.change_table_format,
             "tableformat",
             "\\T",
             "Change the table format used to output results.",
-            aliases=("\\T",),
+            aliases=["\\T"],
             case_sensitive=True,
         )
         special.register_special_command(
@@ -220,12 +223,12 @@ class MyCli(object):
             "redirectformat",
             "\\Tr",
             "Change the table format used to output redirected results.",
-            aliases=("\\Tr",),
+            aliases=["\\Tr"],
             case_sensitive=True,
         )
-        special.register_special_command(self.execute_from_file, "source", "\\. filename", "Execute commands from file.", aliases=("\\.",))
+        special.register_special_command(self.execute_from_file, "source", "\\. filename", "Execute commands from file.", aliases=["\\."])
         special.register_special_command(
-            self.change_prompt_format, "prompt", "\\R", "Change prompt format.", aliases=("\\R",), case_sensitive=True
+            self.change_prompt_format, "prompt", "\\R", "Change prompt format.", aliases=["\\R"], case_sensitive=True
         )
 
     def change_table_format(self, arg, **_):
@@ -478,7 +481,7 @@ class MyCli(object):
                     if password_from_file:
                         new_passwd = password_from_file
                     else:
-                        new_passwd = click.prompt("Password", hide_input=True, show_default=False, type=str, err=True)
+                        new_passwd = click.prompt(f"Password for {user}", hide_input=True, show_default=False, type=str, err=True)
                     self.sqlexecute = SQLExecute(
                         database,
                         user,
@@ -571,7 +574,7 @@ class MyCli(object):
         while special.editor_command(text):
             filename = special.get_filename(text)
             query = special.get_editor_query(text) or self.get_last_query()
-            sql, message = special.open_external_editor(filename, sql=query)
+            sql, message = special.open_external_editor(filename=filename, sql=query)
             if message:
                 # Something went wrong. Raise an exception and bail.
                 raise RuntimeError(message)
@@ -708,11 +711,11 @@ class MyCli(object):
             if not text.strip():
                 return
 
-            if special.is_redirect_command(text):
-                redirect_sql, redirect_operator, redirect_filename = special.get_redirect_components(text)
-                text = redirect_sql
+            if is_redirect_command(text):
+                sql_part, command_part, file_operator_part, file_part = get_redirect_components(text)
+                text = sql_part
                 try:
-                    special.set_redirect(redirect_filename, redirect_operator)
+                    special.set_redirect(command_part, file_operator_part, file_part)
                 except (FileNotFoundError, OSError, RuntimeError) as e:
                     logger.error("sql: %r, error: %r", text, e)
                     logger.error("traceback: %r", traceback.format_exc())
@@ -799,7 +802,7 @@ class MyCli(object):
                     result_count += 1
                     mutating = mutating or destroy or is_mutating(status)
                 special.unset_once_if_written(self.post_redirect_command)
-                special.flush_pipe_once_if_written()
+                special.flush_pipe_once_if_written(self.post_redirect_command)
             except EOFError as e:
                 raise e
             except KeyboardInterrupt:
@@ -1013,6 +1016,11 @@ class MyCli(object):
 
         cnf = self.read_my_cnf_files(self.cnf_files, ["pager", "skip-pager"])
         cnf_pager = cnf["pager"] or self.config["main"]["pager"]
+
+        # help Windows users who haven't edited the default myclirc
+        if WIN and cnf_pager == 'less' and not shutil.which(cnf_pager):
+            cnf_pager = 'more'
+
         if cnf_pager:
             special.set_pager(cnf_pager)
             self.explicit_pager = True
