@@ -1,9 +1,11 @@
-# type: ignore
+from __future__ import annotations
 
-__all__ = []
+from typing import Callable
+
+__all__: list[str] = []
 
 
-def export(defn):
+def export(defn: Callable):
     """Decorator to explicitly mark functions that are exposed in a lib."""
     globals()[defn.__name__] = defn
     __all__.append(defn.__name__)
